class AwsConfigException(Exception):
    pass


class AwsManagerException(Exception):
    pass


class AwsWorkerException(Exception):
    pass


class AwsArgsException(Exception):
    pass


class AwsApiClientException(Exception):
    pass


class AwsResourceCreationFailedException(Exception):
    pass
